using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;

namespace System.Net.PeerToPeer.Collaboration
{
	[Serializable]
	public class PeerEndPointCollection : Collection<PeerEndPoint>, IEquatable<PeerEndPointCollection>
	{
		internal PeerEndPointCollection()
		{
		}

		protected override void SetItem(int index, PeerEndPoint item)
		{
			if (item == null)
			{
				throw new ArgumentNullException("item");
			}
			base.SetItem(index, item);
		}

		protected override void InsertItem(int index, PeerEndPoint item)
		{
			if (item == null)
			{
				throw new ArgumentNullException("item");
			}
			base.InsertItem(index, item);
		}

		public override string ToString()
		{
			bool flag = true;
			StringBuilder stringBuilder = new StringBuilder();
			using (IEnumerator<PeerEndPoint> enumerator = GetEnumerator())
			{
				while (enumerator.MoveNext())
				{
					PeerEndPoint current = enumerator.Current;
					if (!flag)
					{
						stringBuilder.Append(", ");
					}
					else
					{
						flag = false;
					}
					stringBuilder.Append(current.ToString());
				}
			}
			return stringBuilder.ToString();
		}

		public bool Equals(PeerEndPointCollection other)
		{
			bool result = false;
			if (other != null)
			{
				foreach (PeerEndPoint item in other)
				{
					using (IEnumerator<PeerEndPoint> enumerator2 = GetEnumerator())
					{
						while (enumerator2.MoveNext())
						{
							PeerEndPoint current2 = enumerator2.Current;
							if (!item.Equals(current2))
							{
								return result;
							}
						}
					}
				}
				result = true;
			}
			return result;
		}
	}
}
