using Microsoft.Win32.SafeHandles;
using System.Runtime.InteropServices;
using System.Security;
using System.Security.Permissions;

namespace System.Net.PeerToPeer.Collaboration
{
	[SuppressUnmanagedCodeSecurity]
	[SecurityCritical(SecurityCriticalScope.Everything)]
	internal sealed class SafeCollabMemory : SafeHandleZeroOrMinusOneIsInvalid
	{
		private bool allocated;

		internal SafeCollabMemory()
			: base(ownsHandle: true)
		{
		}

		[SecurityPermission(SecurityAction.LinkDemand, UnmanagedCode = true)]
		internal SafeCollabMemory(int cb)
			: base(ownsHandle: true)
		{
			handle = Marshal.AllocHGlobal(cb);
			if (object.Equals(handle, IntPtr.Zero))
			{
				SetHandleAsInvalid();
				throw new PeerToPeerException(SR.GetString("MemoryAllocFailed"));
			}
			allocated = true;
		}

		protected override bool ReleaseHandle()
		{
			if (allocated && !IsInvalid)
			{
				Marshal.FreeHGlobal(handle);
			}
			SetHandleAsInvalid();
			SetHandle(IntPtr.Zero);
			return true;
		}
	}
}
