#define TRACE
using System.Diagnostics;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;

namespace System.Net.PeerToPeer
{
	[Serializable]
	public class PeerNameRecord : ISerializable
	{
		private const int MAX_COMMENT_SIZE = 39;

		private const int MAX_DATA_SIZE = 4096;

		private PeerName m_PeerName;

		private IPEndPointCollection m_EndPointCollection = new IPEndPointCollection();

		private string m_Comment;

		private byte[] m_Data;

		public PeerName PeerName
		{
			get
			{
				return m_PeerName;
			}
			set
			{
				m_PeerName = value;
			}
		}

		public IPEndPointCollection EndPointCollection => m_EndPointCollection;

		public string Comment
		{
			get
			{
				return m_Comment;
			}
			set
			{
				if (value == null)
				{
					throw new ArgumentNullException("Comment", SR.GetString("Pnrp_CommentCantBeNull"));
				}
				if (value.Length <= 0)
				{
					throw new ArgumentException(SR.GetString("Pnrp_CommentCantBeNull"), "Comment");
				}
				if (value.Length > 39)
				{
					throw new ArgumentException(SR.GetString("Pnrp_CommentMaxLengthExceeded", 39));
				}
				m_Comment = value;
			}
		}

		public byte[] Data
		{
			get
			{
				return m_Data;
			}
			set
			{
				if (value == null)
				{
					throw new ArgumentNullException("Data", SR.GetString("Pnrp_DataCantBeNull"));
				}
				if (value.Length <= 0)
				{
					throw new ArgumentException(SR.GetString("Pnrp_DataCantBeNull"), "Data");
				}
				if (value.Length > 4096)
				{
					throw new ArgumentException(SR.GetString("Pnrp_DataLengthExceeded", 4096));
				}
				m_Data = value;
			}
		}

		protected PeerNameRecord(SerializationInfo info, StreamingContext context)
		{
			m_Comment = info.GetString("_Comment");
			m_Data = (info.GetValue("_Data", typeof(byte[])) as byte[]);
			m_EndPointCollection = (info.GetValue("_EndpointList", typeof(IPEndPointCollection)) as IPEndPointCollection);
			m_PeerName = (info.GetValue("_PeerName", typeof(PeerName)) as PeerName);
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_PeerName", m_PeerName);
			info.AddValue("_EndpointList", m_EndPointCollection);
			info.AddValue("_Comment", m_Comment);
			info.AddValue("_Data", m_Data);
		}

		public PeerNameRecord()
		{
		}

		internal void TracePeerNameRecord()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Contents of the PeerNameRecord");
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tPeerName: {0}", PeerName);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tComment: {0}", Comment);
			if (EndPointCollection != null && EndPointCollection.Count != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tThe EndPointList is ");
				foreach (IPEndPoint item in EndPointCollection)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\t\tIPEndPoint is {0}", item);
				}
			}
			else
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tThe EndPointList is empty or null");
			}
			if (Data != null)
			{
				if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Verbose))
				{
					Logging.DumpData(Logging.P2PTraceSource, TraceEventType.Verbose, Logging.P2PTraceSource.MaxDataSize, Data, 0, Data.Length);
					return;
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tCustom data length {0}", Data.Length);
			}
		}
	}
}
