#define TRACE
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;
using System.Threading;

namespace System.Net.PeerToPeer
{
	[Serializable]
	public class PeerToPeerException : Exception, ISerializable
	{
		private const uint FACILITY_P2P = 99u;

		public PeerToPeerException()
		{
		}

		public PeerToPeerException(string message)
			: base(message)
		{
		}

		public PeerToPeerException(string message, Exception innerException)
			: base(message, innerException)
		{
		}

		[SecurityCritical]
		internal static PeerToPeerException CreateFromHr(string message, int hr)
		{
			PeerToPeerException ex = null;
			int num = (hr >> 16) & 0x1FFF;
			IntPtr lpBuffer = IntPtr.Zero;
			try
			{
				if (UnsafeSystemNativeMethods.FormatMessage((FormatMessageFlags)(0x2100 | (((long)num == 99) ? 2048 : 4096)), ((long)num == 99) ? PeerToPeerOSHelper.P2PModuleHandle : IntPtr.Zero, (uint)hr, 0u, ref lpBuffer, 0u, IntPtr.Zero) != 0)
				{
					string message2 = Marshal.PtrToStringUni(lpBuffer);
					ex = new PeerToPeerException(message, new Win32Exception(hr, message2));
				}
				else
				{
					ex = new PeerToPeerException(message, Marshal.GetExceptionForHR(hr));
				}
			}
			catch (Exception ex2)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Warning, 0, "Could not get the error message for error code {0} - Exception {1}", hr, ex2);
				if (ex2 is ThreadAbortException || ex2 is StackOverflowException || ex2 is OutOfMemoryException)
				{
					throw;
				}
			}
			finally
			{
				if (lpBuffer != IntPtr.Zero)
				{
					UnsafeSystemNativeMethods.LocalFree(lpBuffer);
				}
			}
			if (ex == null)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Warning, 0, "Could not get the error message for error code {0}", hr);
				ex = new PeerToPeerException(message + "Underlying native error " + hr);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "Exception: {0}", ex);
			return ex;
		}

		protected PeerToPeerException(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			GetObjectData(info, context);
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter)]
		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);
		}
	}
}
