using System.Security;
using System.Security.Permissions;

namespace System.Net.PeerToPeer
{
	internal static class PeerToPeerOSHelper
	{
		private static bool s_supportsP2P;

		private static SafeLoadLibrary s_P2PLibrary;

		internal static bool SupportsP2P => s_supportsP2P;

		internal static IntPtr P2PModuleHandle
		{
			[SecurityCritical]
			[SecurityPermission(SecurityAction.LinkDemand, UnmanagedCode = true)]
			get
			{
				if (!s_P2PLibrary.IsClosed && !s_P2PLibrary.IsInvalid)
				{
					return s_P2PLibrary.DangerousGetHandle();
				}
				return IntPtr.Zero;
			}
		}

		[SecurityCritical]
		static PeerToPeerOSHelper()
		{
			s_supportsP2P = false;
			s_P2PLibrary = null;
			s_P2PLibrary = SafeLoadLibrary.LoadLibraryEx("p2p.dll");
			if (!s_P2PLibrary.IsInvalid)
			{
				IntPtr procAddress = UnsafeSystemNativeMethods.GetProcAddress(s_P2PLibrary, "PeerCreatePeerName");
				if (procAddress != IntPtr.Zero)
				{
					s_supportsP2P = true;
				}
			}
		}
	}
}
