using System.Net.Sockets;

namespace System.Net.PeerToPeer
{
	internal static class SystemNetHelpers
	{
		internal const int IPv6AddressSize = 28;

		internal const int IPv4AddressSize = 16;

		internal static byte[] SOCKADDRFromIPEndPoint(IPEndPoint ipEndPoint)
		{
			byte[] array = new byte[(ipEndPoint.AddressFamily == AddressFamily.InterNetworkV6) ? 28 : 16];
			array[0] = (byte)ipEndPoint.AddressFamily;
			array[1] = (byte)((int)ipEndPoint.AddressFamily >> 8);
			if (ipEndPoint.AddressFamily == AddressFamily.InterNetworkV6)
			{
				array[2] = (byte)(ipEndPoint.Port >> 8);
				array[3] = (byte)ipEndPoint.Port;
				array[4] = 0;
				array[5] = 0;
				array[6] = 0;
				array[7] = 0;
				long scopeId = ipEndPoint.Address.ScopeId;
				array[24] = (byte)scopeId;
				array[25] = (byte)(scopeId >> 8);
				array[26] = (byte)(scopeId >> 16);
				array[27] = (byte)(scopeId >> 24);
				byte[] addressBytes = ipEndPoint.Address.GetAddressBytes();
				for (int i = 0; i < addressBytes.Length; i++)
				{
					array[8 + i] = addressBytes[i];
				}
			}
			else
			{
				array[2] = (byte)(ipEndPoint.Port >> 8);
				array[3] = (byte)ipEndPoint.Port;
				byte[] addressBytes = ipEndPoint.Address.GetAddressBytes();
				for (int i = 0; i < addressBytes.Length; i++)
				{
					array[4 + i] = addressBytes[i];
				}
			}
			return array;
		}

		internal static IPEndPoint IPEndPointFromSOCKADDRBuffer(byte[] buffer)
		{
			IPAddress iPAddress = null;
			int num = 0;
			num = buffer[0] + (buffer[1] << 8);
			int port = buffer[3] + (buffer[2] << 8);
			switch (num)
			{
			case 2:
			{
				byte[] address = new byte[4]
				{
					buffer[4],
					buffer[5],
					buffer[6],
					buffer[7]
				};
				iPAddress = new IPAddress(address);
				break;
			}
			case 23:
			{
				byte[] array = new byte[16];
				for (int i = 0; i < 16; i++)
				{
					array[i] = buffer[8 + i];
				}
				long scopeId = (long)(buffer[24] + ((ulong)buffer[25] << 8) + ((ulong)buffer[26] << 16) + ((ulong)buffer[27] << 24));
				iPAddress = new IPAddress(array);
				iPAddress.ScopeId = scopeId;
				break;
			}
			}
			return new IPEndPoint(iPAddress, port);
		}
	}
}
