using Microsoft.Win32.SafeHandles;
using System.Runtime.ConstrainedExecution;
using System.Runtime.InteropServices;
using System.Security;
using System.Threading;

namespace System.Net.PeerToPeer
{
	[SuppressUnmanagedCodeSecurity]
	internal static class UnsafeP2PNativeMethods
	{
		internal const string P2P = "p2p.dll";

		private const int PNRP_VERSION = 2;

		private static object s_InternalSyncObject;

		private static bool s_Initialized;

		private static object InternalSyncObject
		{
			get
			{
				if (s_InternalSyncObject == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref s_InternalSyncObject, value, null);
				}
				return s_InternalSyncObject;
			}
		}

		[DllImport("p2p.dll", CharSet = CharSet.Unicode)]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.Success)]
		internal static extern void PeerFreeData(IntPtr dataToFree);

		[DllImport("p2p.dll", CharSet = CharSet.Unicode)]
		internal static extern int PeerPnrpGetCloudInfo(out uint pNumClouds, out SafePeerData pArrayOfClouds);

		[DllImport("p2p.dll", CharSet = CharSet.Unicode)]
		internal static extern int PeerPnrpStartup(ushort versionRequired);

		[DllImport("p2p.dll", CharSet = CharSet.Unicode)]
		internal static extern int PeerCreatePeerName(string identity, string classfier, out SafePeerData peerName);

		[DllImport("p2p.dll", CharSet = CharSet.Unicode)]
		internal static extern int PeerIdentityGetDefault(out SafePeerData defaultIdentity);

		[DllImport("p2p.dll", CharSet = CharSet.Unicode)]
		internal static extern int PeerNameToPeerHostName(string peerName, out SafePeerData peerHostName);

		[DllImport("p2p.dll", CharSet = CharSet.Unicode)]
		internal static extern int PeerHostNameToPeerName(string peerHostName, out SafePeerData peerName);

		[DllImport("p2p.dll", CharSet = CharSet.Unicode)]
		public static extern int PeerPnrpRegister(string pcwzPeerName, ref PEER_PNRP_REGISTRATION_INFO registrationInfo, out SafePeerNameUnregister handle);

		[DllImport("p2p.dll", CharSet = CharSet.Unicode)]
		public static extern int PeerPnrpUnregister(IntPtr handle);

		[DllImport("p2p.dll", CharSet = CharSet.Unicode)]
		[SecurityCritical]
		public static extern int PeerPnrpUpdateRegistration(SafePeerNameUnregister hRegistration, ref PEER_PNRP_REGISTRATION_INFO registrationInfo);

		[DllImport("p2p.dll", CharSet = CharSet.Unicode)]
		public static extern int PeerPnrpResolve(string pcwzPeerNAme, string pcwzCloudName, ref uint pcEndPoints, out SafePeerData pEndPoints);

		[DllImport("p2p.dll", CharSet = CharSet.Unicode)]
		public static extern int PeerPnrpStartResolve(string pcwzPeerNAme, string pcwzCloudName, uint cEndPoints, SafeWaitHandle hEvent, out SafePeerNameEndResolve safePeerNameEndResolve);

		[DllImport("p2p.dll", CharSet = CharSet.Unicode)]
		public static extern int PeerPnrpGetEndpoint(IntPtr Handle, out SafePeerData pEndPoint);

		[DllImport("p2p.dll", CharSet = CharSet.Unicode)]
		public static extern int PeerPnrpEndResolve(IntPtr Handle);

		[SecurityCritical]
		internal static void PnrpStartup()
		{
			if (s_Initialized)
			{
				return;
			}
			lock (InternalSyncObject)
			{
				if (!s_Initialized)
				{
					int num = PeerPnrpStartup(2);
					if (num != 0)
					{
						throw new PeerToPeerException(SR.GetString("Pnrp_StartupFailed"), Marshal.GetExceptionForHR(num));
					}
					s_Initialized = true;
				}
			}
		}
	}
}
