using System.Diagnostics;
using System.Globalization;
using System.Threading;

namespace System.Net
{
	internal class P2PTraceSource : TraceSource
	{
		private const string P2PTraceSourceName = "System.Net.PeerToPeer";

		private const int DefaultMaxDataSize = 1024;

		private const string AttributeNameMaxDataSize = "maxdatasize";

		private static readonly string[] P2PTraceSourceSupportedAttributes = new string[1]
		{
			"maxdatasize"
		};

		private int m_maxDataSize = 1024;

		internal int MaxDataSize => m_maxDataSize;

		internal P2PTraceSource()
			: base("System.Net.PeerToPeer")
		{
			if (!base.Attributes.ContainsKey("maxdatasize"))
			{
				return;
			}
			try
			{
				m_maxDataSize = int.Parse(base.Attributes["maxdatasize"], NumberFormatInfo.InvariantInfo);
			}
			catch (Exception ex)
			{
				if (ex is ThreadAbortException || ex is StackOverflowException || ex is OutOfMemoryException)
				{
					throw;
				}
			}
		}

		protected override string[] GetSupportedAttributes()
		{
			return P2PTraceSourceSupportedAttributes;
		}
	}
}
