﻿using System;
using System.Diagnostics;
using System.Globalization;
using System.Threading;

namespace System.Net
{
	internal class P2PTraceSource : TraceSource
	{
		internal P2PTraceSource() : base("System.Net.PeerToPeer")
		{
			if (base.Attributes.ContainsKey("maxdatasize"))
			{
				try
				{
					this.m_maxDataSize = int.Parse(base.Attributes["maxdatasize"], NumberFormatInfo.InvariantInfo);
				}
				catch (Exception ex)
				{
					if (ex is ThreadAbortException || ex is StackOverflowException || ex is OutOfMemoryException)
					{
						throw;
					}
				}
			}
		}

		protected override string[] GetSupportedAttributes()
		{
			return P2PTraceSource.P2PTraceSourceSupportedAttributes;
		}

		internal int MaxDataSize
		{
			get
			{
				return this.m_maxDataSize;
			}
		}

		private const string P2PTraceSourceName = "System.Net.PeerToPeer";

		private const int DefaultMaxDataSize = 1024;

		private const string AttributeNameMaxDataSize = "maxdatasize";

		private static readonly string[] P2PTraceSourceSupportedAttributes = new string[]
		{
			"maxdatasize"
		};

		private int m_maxDataSize = 1024;
	}
}
