﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Net.Mail;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Cryptography.X509Certificates;
using System.Security.Permissions;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	[Serializable]
	public class PeerContact : Peer, IEquatable<PeerContact>, ISerializable
	{
		public PeerName PeerName
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_peerName;
			}
			internal set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_peerName = value;
			}
		}

		public string Nickname
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_nickname;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_nickname = value;
			}
		}

		public string DisplayName
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_displayName;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_displayName = value;
			}
		}

		public MailAddress EmailAddress
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_emailAddress;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_emailAddress = value;
			}
		}

		public SubscriptionType SubscribeAllowed
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.InternalSubscribeAllowedGet();
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.InternalSubscribeAllowedSet(value);
			}
		}

		internal virtual SubscriptionType InternalSubscribeAllowedGet()
		{
			return this.m_subscribeAllowed;
		}

		internal virtual void InternalSubscribeAllowedSet(SubscriptionType value)
		{
			this.m_subscribeAllowed = value;
		}

		public bool IsSubscribed
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.InternalIsSubscribed();
			}
			internal set
			{
				this.m_isSubscribed = value;
			}
		}

		internal virtual bool InternalIsSubscribed()
		{
			bool isSubscribed;
			lock (this.IsSubscribeLock)
			{
				isSubscribed = this.m_isSubscribed;
			}
			return isSubscribed;
		}

		internal object IsSubscribeLock
		{
			get
			{
				if (this.m_isSubscribeLock == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref this.m_isSubscribeLock, value, null);
				}
				return this.m_isSubscribeLock;
			}
		}

		public X509Certificate2 Credentials
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				return this.m_credentials;
			}
			internal set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_credentials = value;
			}
		}

		internal bool JustCreated
		{
			get
			{
				return this.m_justCreated;
			}
			set
			{
				this.m_justCreated = value;
			}
		}

		internal string ContactXml
		{
			get
			{
				return this.m_contactXml;
			}
			set
			{
				this.m_contactXml = value;
			}
		}

		public unsafe override PeerEndPointCollection PeerEndPoints
		{
			[SecurityCritical]
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Get PeerEndPoints called.");
				PeerEndPointCollection peerEndPointCollection = new PeerEndPointCollection();
				SafeCollabEnum safeCollabEnum = null;
				uint num = 0U;
				int num2 = 0;
				try
				{
					SafeCollabMemory safeCollabMemory = null;
					try
					{
						PEER_CONTACT peer_CONTACT = CollaborationHelperFunctions.ConvertPeerContactToPEER_CONTACT(this, ref safeCollabMemory);
						num2 = UnsafeCollabNativeMethods.PeerCollabEnumEndpoints(ref peer_CONTACT, out safeCollabEnum);
					}
					finally
					{
						if (safeCollabMemory != null)
						{
							safeCollabMemory.Dispose();
						}
					}
					if (num2 != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabEnumEndpoints returned with errorcode {0}", new object[]
						{
							num2
						});
						return peerEndPointCollection;
					}
					num2 = UnsafeCollabNativeMethods.PeerGetItemCount(safeCollabEnum, ref num);
					if (num2 != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetItemCount returned with errorcode {0}", new object[]
						{
							num2
						});
						return peerEndPointCollection;
					}
					if (num == 0U)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "No endpoints. Get PeerEndPoints returning.");
						return peerEndPointCollection;
					}
					SafeCollabData safeCollabData;
					num2 = UnsafeCollabNativeMethods.PeerGetNextItem(safeCollabEnum, ref num, out safeCollabData);
					IntPtr value = safeCollabData.DangerousGetHandle();
					IntPtr* ptr = (IntPtr*)((void*)value);
					for (ulong num3 = 0UL; num3 < (ulong)num; num3 += 1UL)
					{
						IntPtr ptr2 = ptr[num3 * (ulong)((long)sizeof(IntPtr)) / (ulong)sizeof(IntPtr)];
						PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(ptr2, typeof(PEER_ENDPOINT));
						PeerEndPoint item = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
						peerEndPointCollection.Add(item);
					}
				}
				finally
				{
					if (safeCollabEnum != null)
					{
						safeCollabEnum.Dispose();
					}
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving Get PeerEndPoints with {0} endpoints.", new object[]
				{
					peerEndPointCollection.Count
				});
				return peerEndPointCollection;
			}
		}

		[SecurityCritical]
		static PeerContact()
		{
			CollaborationHelperFunctions.Initialize();
		}

		[SecurityCritical]
		internal PeerContact()
		{
			this.OnSubscribeCompletedDelegate = new SendOrPostCallback(this.SubscribeCompletedWaitCallback);
		}

		[SecurityCritical]
		protected PeerContact(SerializationInfo serializationInfo, StreamingContext streamingContext) : this()
		{
			this.m_peerName = (PeerName)serializationInfo.GetValue("_PeerName", typeof(PeerName));
			this.m_nickname = serializationInfo.GetString("_NickName");
			this.m_displayName = serializationInfo.GetString("_DisplayName");
			try
			{
				this.m_emailAddress = new MailAddress(serializationInfo.GetString("_EmailAddress"));
			}
			catch (SerializationException)
			{
			}
			this.m_subscribeAllowed = (SubscriptionType)serializationInfo.GetValue("_SubscribeAllowed", typeof(SubscriptionType));
			byte[] rawData = (byte[])serializationInfo.GetValue("_Credentials", typeof(byte[]));
			this.m_credentials = new X509Certificate2(rawData);
			this.m_contactXml = serializationInfo.GetString("_ContactXml");
			this.m_justCreated = serializationInfo.GetBoolean("_JustCreated");
		}

		[SecurityCritical]
		public string ToXml()
		{
			string text = null;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering ToXml().");
			string result;
			if (this.JustCreated)
			{
				result = this.ContactXml;
			}
			else
			{
				int num = UnsafeCollabNativeMethods.PeerCollabExportContact(this.PeerName.ToString(), ref text);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabExportContact returned with errorcode {0}.", new object[]
					{
						num
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ContactToXmlFailed"), num);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ToXml() with XML string: {0}", new object[]
				{
					text
				});
				result = text;
			}
			return result;
		}

		[SecurityCritical]
		public static PeerContact FromXml(string peerContactXml)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering FromXml() with XML string: {0}", new object[]
			{
				peerContactXml
			});
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			CollaborationHelperFunctions.Initialize();
			if (peerContactXml == null)
			{
				throw new ArgumentNullException("peerContactXml");
			}
			SafeCollabData safeCollabData = null;
			PeerContact peerContact = null;
			int num;
			try
			{
				num = UnsafeCollabNativeMethods.PeerCollabParseContact(peerContactXml, out safeCollabData);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabParseContact returned with errorcode {0}. Contact already exists.", new object[]
					{
						num
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ContactFromXmlFailed"), num);
				}
				PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_CONTACT));
				peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
			}
			finally
			{
				if (safeCollabData != null)
				{
					safeCollabData.Dispose();
				}
			}
			try
			{
				safeCollabData = null;
				num = UnsafeCollabNativeMethods.PeerCollabGetContact(peerContact.PeerName.ToString(), out safeCollabData);
			}
			finally
			{
				if (safeCollabData != null)
				{
					safeCollabData.Dispose();
				}
			}
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "Error or contact not found in Contact Manager. ErrorCode {0}", new object[]
				{
					num
				});
				peerContact.JustCreated = true;
				peerContact.ContactXml = peerContactXml;
			}
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving FromXml() with following peercontact");
				peerContact.TracePeerContact();
			}
			return peerContact;
		}

		public virtual void Subscribe()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering Subscribe().");
			if (this.IsSubscribed)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Already subscribed. Leaving Subscribe().");
			}
			else
			{
				this.InternalSubscribe(false);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving Subscribe().");
			}
		}

		private void InternalSubscribe(object isAsyncObj)
		{
			bool flag = (bool)isAsyncObj;
			Exception ex = null;
			lock (this.IsSubscribeLock)
			{
				this.IsSubscribed = true;
				try
				{
					PeerCollaboration.ContactManager.UpdateContact(this);
				}
				catch (Exception ex2)
				{
					this.IsSubscribed = false;
					if (!flag)
					{
						throw;
					}
					ex = ex2;
				}
			}
			if (flag)
			{
				SubscribeCompletedEventArgs args;
				if (ex == null)
				{
					args = new SubscribeCompletedEventArgs(null, this, null, false, this.AsyncOp.UserSuppliedState);
				}
				else
				{
					args = new SubscribeCompletedEventArgs(null, null, ex, false, this.AsyncOp.UserSuppliedState);
				}
				this.PrepareToRaiseSubscribeCompletedEvent(this.AsyncOp, args);
			}
		}

		internal AsyncOperation AsyncOp
		{
			get
			{
				return this.m_AsyncOp;
			}
			set
			{
				this.m_AsyncOp = value;
			}
		}

		internal object AsyncLock
		{
			get
			{
				if (this.m_asyncOpLock == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref this.m_asyncOpLock, value, null);
				}
				return this.m_asyncOpLock;
			}
		}

		private event EventHandler<SubscribeCompletedEventArgs> m_subscribeCompleted;

		public event EventHandler<SubscribeCompletedEventArgs> SubscribeCompleted
		{
			add
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.m_subscribeCompleted = (EventHandler<SubscribeCompletedEventArgs>)Delegate.Combine(this.m_subscribeCompleted, value);
			}
			remove
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.m_subscribeCompleted = (EventHandler<SubscribeCompletedEventArgs>)Delegate.Remove(this.m_subscribeCompleted, value);
			}
		}

		public virtual void SubscribeAsync(object userToken)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			if (userToken == null)
			{
				throw new ArgumentNullException("userToken");
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering SubscribeAsync() with user token {0}.", new object[]
			{
				userToken
			});
			lock (this.AsyncLock)
			{
				if (this.AsyncOp != null)
				{
					throw new PeerToPeerException(SR.GetString("Collab_DuplicateSubscribeAsync"));
				}
				this.AsyncOp = AsyncOperationManager.CreateOperation(userToken);
			}
			ThreadPool.QueueUserWorkItem(new WaitCallback(this.InternalSubscribe), true);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving SubscribeAsync().");
		}

		protected void OnSubscribeCompleted(SubscribeCompletedEventArgs e)
		{
			EventHandler<SubscribeCompletedEventArgs> subscribeCompleted = this.m_subscribeCompleted;
			if (subscribeCompleted != null)
			{
				subscribeCompleted(this, e);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the subscribe completed event callback.");
			}
		}

		private void SubscribeCompletedWaitCallback(object operationState)
		{
			this.AsyncOp = null;
			this.OnSubscribeCompleted((SubscribeCompletedEventArgs)operationState);
		}

		internal void PrepareToRaiseSubscribeCompletedEvent(AsyncOperation asyncOP, SubscribeCompletedEventArgs args)
		{
			asyncOP.PostOperationCompleted(this.OnSubscribeCompletedDelegate, args);
		}

		public virtual void Unsubscribe()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			if (this.IsSubscribed)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering UnSubscribe().");
				lock (this.IsSubscribeLock)
				{
					this.IsSubscribed = false;
					try
					{
						PeerCollaboration.ContactManager.UpdateContact(this);
					}
					catch (Exception)
					{
						this.IsSubscribed = true;
						throw;
					}
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Laaving UnSubscribe().");
			}
		}

		[SecurityCritical]
		public override PeerInvitationResponse Invite()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			Guid currentApplicationGuid = Peer.CurrentApplicationGuid;
			if (currentApplicationGuid.Equals(Guid.Empty))
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoGuidForCurrApp"));
			}
			return this.InternalInviteAllEndPoints(this.PeerEndPoints, null, null, currentApplicationGuid);
		}

		[SecurityCritical]
		public override PeerInvitationResponse Invite(PeerApplication applicationToInvite, string message, byte[] invitationData)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			if (applicationToInvite == null)
			{
				throw new ArgumentNullException("applicationToInvite");
			}
			if (applicationToInvite.Id == Guid.Empty)
			{
				throw new PeerToPeerException(SR.GetString("Collab_EmptyGuidError"));
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			return this.InternalInviteAllEndPoints(this.PeerEndPoints, message, invitationData, applicationToInvite.Id);
		}

		[SecurityCritical]
		private PeerInvitationResponse InternalInviteAllEndPoints(PeerEndPointCollection peerEndPoints, string message, byte[] invitationData, Guid applicationId)
		{
			if (peerEndPoints == null || peerEndPoints.Count == 0)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			bool flag = false;
			bool flag2 = false;
			foreach (PeerEndPoint peerEndPoint in peerEndPoints)
			{
				PeerInvitationResponse peerInvitationResponse = Peer.InternalInviteEndPoint(applicationId, message, invitationData, peerEndPoint, this);
				if (peerInvitationResponse.PeerInvitationResponseType == PeerInvitationResponseType.Accepted)
				{
					return peerInvitationResponse;
				}
				if (peerInvitationResponse.PeerInvitationResponseType == PeerInvitationResponseType.Declined)
				{
					flag = true;
				}
				else if (peerInvitationResponse.PeerInvitationResponseType == PeerInvitationResponseType.Expired)
				{
					flag2 = true;
				}
			}
			PeerInvitationResponse result;
			if (flag)
			{
				result = new PeerInvitationResponse(PeerInvitationResponseType.Declined);
			}
			else
			{
				if (!flag2)
				{
					throw new PeerToPeerException(SR.GetString("Collab_InviteFailed"));
				}
				result = new PeerInvitationResponse(PeerInvitationResponseType.Expired);
			}
			return result;
		}

		[SecurityCritical]
		public PeerInvitationResponse Invite(PeerEndPoint peerEndPoint)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			if (peerEndPoint.EndPoint == null)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndPointInPeerEndPoint"));
			}
			Guid currentApplicationGuid = Peer.CurrentApplicationGuid;
			if (currentApplicationGuid.Equals(Guid.Empty))
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoGuidForCurrApp"));
			}
			PeerInvitationResponse peerInvitationResponse = Peer.InternalInviteEndPoint(currentApplicationGuid, null, null, peerEndPoint, this);
			CollaborationHelperFunctions.ThrowIfInvitationResponseInvalid(peerInvitationResponse);
			return peerInvitationResponse;
		}

		[SecurityCritical]
		public PeerInvitationResponse Invite(PeerEndPoint peerEndPoint, PeerApplication applicationToInvite, string message, byte[] invitationData)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (applicationToInvite == null)
			{
				throw new ArgumentNullException("applicationToInvite");
			}
			if (applicationToInvite.Id == Guid.Empty)
			{
				throw new PeerToPeerException(SR.GetString("Collab_EmptyGuidError"));
			}
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			if (peerEndPoint.EndPoint == null)
			{
				throw new ArgumentException(SR.GetString("Collab_NoEndPointInPeerEndPoint"));
			}
			PeerInvitationResponse peerInvitationResponse = Peer.InternalInviteEndPoint(applicationToInvite.Id, message, invitationData, peerEndPoint, this);
			CollaborationHelperFunctions.ThrowIfInvitationResponseInvalid(peerInvitationResponse);
			return peerInvitationResponse;
		}

		[SecurityCritical]
		public override void InviteAsync(object userToken)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (userToken == null)
			{
				throw new ArgumentException(SR.GetString("NullUserToken"));
			}
			Guid currentApplicationGuid = Peer.CurrentApplicationGuid;
			if (currentApplicationGuid.Equals(Guid.Empty))
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoGuidForCurrApp"));
			}
			PeerEndPointCollection peerEndPoints = this.PeerEndPoints;
			if (peerEndPoints == null || peerEndPoints.Count == 0)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			base.InternalInviteAsync(currentApplicationGuid, null, null, peerEndPoints, this, userToken);
		}

		[SecurityCritical]
		public override void InviteAsync(PeerApplication applicationToInvite, string message, byte[] invitationData, object userToken)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (userToken == null)
			{
				throw new ArgumentException(SR.GetString("NullUserToken"));
			}
			if (applicationToInvite == null)
			{
				throw new ArgumentNullException("applicationToInvite");
			}
			if (applicationToInvite.Id == Guid.Empty)
			{
				throw new PeerToPeerException(SR.GetString("Collab_EmptyGuidError"));
			}
			PeerEndPointCollection peerEndPoints = this.PeerEndPoints;
			if (peerEndPoints == null || peerEndPoints.Count == 0)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			base.InternalInviteAsync(applicationToInvite.Id, null, null, peerEndPoints, this, userToken);
		}

		[SecurityCritical]
		public void InviteAsync(PeerEndPoint peerEndPoint, object userToken)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (userToken == null)
			{
				throw new ArgumentException(SR.GetString("NullUserToken"));
			}
			Guid currentApplicationGuid = Peer.CurrentApplicationGuid;
			if (currentApplicationGuid.Equals(Guid.Empty))
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoGuidForCurrApp"));
			}
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			if (peerEndPoint.EndPoint == null)
			{
				throw new ArgumentException(SR.GetString("Collab_NoEndPointInPeerEndPoint"));
			}
			PeerEndPointCollection peerEndPointCollection = new PeerEndPointCollection();
			peerEndPointCollection.Add(peerEndPoint);
			base.InternalInviteAsync(currentApplicationGuid, null, null, peerEndPointCollection, this, userToken);
		}

		[SecurityCritical]
		public void InviteAsync(PeerEndPoint peerEndPoint, string message, byte[] invitationData, PeerApplication applicationToInvite, object userToken)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (userToken == null)
			{
				throw new ArgumentException(SR.GetString("NullUserToken"));
			}
			if (applicationToInvite == null)
			{
				throw new ArgumentNullException("applicationToInvite");
			}
			if (applicationToInvite.Id == Guid.Empty)
			{
				throw new PeerToPeerException(SR.GetString("Collab_EmptyGuidError"));
			}
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			if (peerEndPoint.EndPoint == null)
			{
				throw new ArgumentException(SR.GetString("Collab_NoEndPointInPeerEndPoint"));
			}
			PeerEndPointCollection peerEndPointCollection = new PeerEndPointCollection();
			peerEndPointCollection.Add(peerEndPoint);
			base.InternalInviteAsync(applicationToInvite.Id, message, invitationData, peerEndPointCollection, this, userToken);
		}

		[SecurityCritical]
		public PeerApplicationCollection GetApplications()
		{
			return this.InternalGetAllApplications(Guid.Empty, false);
		}

		[SecurityCritical]
		public PeerApplicationCollection GetApplications(Guid applicationId)
		{
			return this.InternalGetAllApplications(applicationId, true);
		}

		[SecurityCritical]
		public PeerApplicationCollection GetApplications(PeerEndPoint peerEndPoint)
		{
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			return PeerContact.InternalGetApplications(Guid.Empty, false, peerEndPoint);
		}

		[SecurityCritical]
		public PeerApplicationCollection GetApplications(PeerEndPoint peerEndPoint, Guid applicationId)
		{
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			return PeerContact.InternalGetApplications(applicationId, true, peerEndPoint);
		}

		[SecurityCritical]
		private PeerApplicationCollection InternalGetAllApplications(Guid applicationId, bool guidSupplied)
		{
			Dictionary<Guid, PeerApplication> dictionary = new Dictionary<Guid, PeerApplication>();
			PeerEndPointCollection peerEndPoints = this.PeerEndPoints;
			PeerApplicationCollection peerApplicationCollection;
			foreach (PeerEndPoint peerEndPoint in peerEndPoints)
			{
				peerApplicationCollection = PeerContact.InternalGetApplications(applicationId, guidSupplied, peerEndPoint);
				foreach (PeerApplication peerApplication in peerApplicationCollection)
				{
					dictionary[peerApplication.Id] = peerApplication;
				}
			}
			Dictionary<Guid, PeerApplication>.ValueCollection values = dictionary.Values;
			peerApplicationCollection = new PeerApplicationCollection();
			foreach (PeerApplication peerApplication in values)
			{
				peerApplicationCollection.Add(peerApplication);
			}
			return peerApplicationCollection;
		}

		[SecurityCritical]
		private unsafe static PeerApplicationCollection InternalGetApplications(Guid applicationId, bool guidSupplied, PeerEndPoint peerEndPoint)
		{
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering InternalGetApplications() with the following PeerEndPoint");
				peerEndPoint.TracePeerEndPoint();
			}
			PeerApplicationCollection peerApplicationCollection = new PeerApplicationCollection();
			SafeCollabEnum safeCollabEnum = null;
			uint num = 0U;
			GCHandle gchandle = default(GCHandle);
			IntPtr pObjectId = IntPtr.Zero;
			if (guidSupplied)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Guid supplied is {0}.", new object[]
				{
					applicationId.ToString()
				});
				GUID guid = CollaborationHelperFunctions.ConvertGuidToGUID(applicationId);
				gchandle = GCHandle.Alloc(guid, GCHandleType.Pinned);
				pObjectId = gchandle.AddrOfPinnedObject();
			}
			PEER_ENDPOINT peer_ENDPOINT = default(PEER_ENDPOINT);
			peer_ENDPOINT.peerAddress = CollaborationHelperFunctions.ConvertIPEndpointToPEER_ADDRESS(peerEndPoint.EndPoint);
			GCHandle gchandle2 = GCHandle.Alloc(peerEndPoint.Name, GCHandleType.Pinned);
			peer_ENDPOINT.pwzEndpointName = gchandle2.AddrOfPinnedObject();
			GCHandle gchandle3 = GCHandle.Alloc(peer_ENDPOINT, GCHandleType.Pinned);
			IntPtr pcEndpoint = gchandle3.AddrOfPinnedObject();
			try
			{
				int num2 = UnsafeCollabNativeMethods.PeerCollabEnumApplications(pcEndpoint, pObjectId, out safeCollabEnum);
				if (num2 != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabEnumApplications returned with errorcode {0}", new object[]
					{
						num2
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetAppsFailed"), num2);
				}
				num2 = UnsafeCollabNativeMethods.PeerGetItemCount(safeCollabEnum, ref num);
				if (num2 != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetItemCount returned with errorcode {0}", new object[]
					{
						num2
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetAppsFailed"), num2);
				}
				if (num == 0U)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "No PeerApplications found.");
					return peerApplicationCollection;
				}
				SafeCollabData safeCollabData;
				num2 = UnsafeCollabNativeMethods.PeerGetNextItem(safeCollabEnum, ref num, out safeCollabData);
				if (num2 != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
					{
						num2
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetAppsFailed"), num2);
				}
				IntPtr value = safeCollabData.DangerousGetHandle();
				IntPtr* ptr = (IntPtr*)((void*)value);
				for (ulong num3 = 0UL; num3 < (ulong)num; num3 += 1UL)
				{
					PEER_APPLICATION* ptr2 = (PEER_APPLICATION*)((void*)ptr[num3 * (ulong)((long)sizeof(IntPtr)) / (ulong)sizeof(IntPtr)]);
					string description = Marshal.PtrToStringUni(ptr2->pwzDescription);
					byte[] array = null;
					if (ptr2->data.cbData != 0U)
					{
						array = new byte[ptr2->data.cbData];
						Marshal.Copy(ptr2->data.pbData, array, 0, (int)ptr2->data.cbData);
					}
					PeerApplication peerApplication = new PeerApplication(CollaborationHelperFunctions.ConvertGUIDToGuid(ptr2->guid), description, array, null, null, PeerScope.None);
					if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Retrieved following Application");
						peerApplication.TracePeerApplication();
					}
					peerApplicationCollection.Add(peerApplication);
				}
			}
			finally
			{
				if (gchandle.IsAllocated)
				{
					gchandle.Free();
				}
				if (gchandle2.IsAllocated)
				{
					gchandle2.Free();
				}
				if (gchandle3.IsAllocated)
				{
					gchandle3.Free();
				}
				if (safeCollabEnum != null)
				{
					safeCollabEnum.Dispose();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving InternalGetApplications(). Returning collection with {0} applications.", new object[]
			{
				peerApplicationCollection.Count
			});
			return peerApplicationCollection;
		}

		[SecurityCritical]
		public PeerObjectCollection GetObjects(PeerEndPoint peerEndPoint)
		{
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			return Peer.InternalGetObjects(Guid.Empty, false, peerEndPoint);
		}

		[SecurityCritical]
		public PeerObjectCollection GetObjects(PeerEndPoint peerEndPoint, Guid objectId)
		{
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			return Peer.InternalGetObjects(objectId, true, peerEndPoint);
		}

		internal override void RefreshIfNeeded()
		{
		}

		private event EventHandler<ApplicationChangedEventArgs> m_applicationChanged;

		public event EventHandler<ApplicationChangedEventArgs> ApplicationChanged
		{
			[SecurityCritical]
			add
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.AddApplicationChanged(value);
			}
			[SecurityCritical]
			remove
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.RemoveApplicationChanged(value);
			}
		}

		internal object LockAppChangedEvent
		{
			get
			{
				if (this.m_lockAppChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref this.m_lockAppChangedEvent, value, null);
				}
				return this.m_lockAppChangedEvent;
			}
		}

		internal RegisteredWaitHandle AppChangedWaitHandle
		{
			set
			{
				this.m_regAppChangedWaitHandle = value;
			}
		}

		internal AutoResetEvent AppChangedEvent
		{
			get
			{
				return this.m_appChangedEvent;
			}
			set
			{
				this.m_appChangedEvent = value;
			}
		}

		[SecurityCritical]
		internal virtual void AddApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddApplicationChanged().");
			lock (this.LockAppChangedEvent)
			{
				if (this.m_applicationChanged == null)
				{
					this.m_appChangedEvent = new AutoResetEvent(false);
					this.AppChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(this.m_appChangedEvent, new WaitOrTimerCallback(this.ApplicationChangedCallback), null, -1, false);
					PEER_COLLAB_EVENT_REGISTRATION peer_COLLAB_EVENT_REGISTRATION = default(PEER_COLLAB_EVENT_REGISTRATION);
					peer_COLLAB_EVENT_REGISTRATION.eventType = PeerCollabEventType.EndPointApplicationChanged;
					peer_COLLAB_EVENT_REGISTRATION.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(this.m_appChangedEvent.SafeWaitHandle, 1U, ref peer_COLLAB_EVENT_REGISTRATION, out this.m_safeAppChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ApplicationChangedRegFailed"), num);
					}
				}
				this.m_applicationChanged = (EventHandler<ApplicationChangedEventArgs>)Delegate.Combine(this.m_applicationChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddApplicationChanged() successful.");
		}

		[SecurityCritical]
		internal virtual void RemoveApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() called.");
			lock (this.LockAppChangedEvent)
			{
				this.m_applicationChanged = (EventHandler<ApplicationChangedEventArgs>)Delegate.Remove(this.m_applicationChanged, callback);
				if (this.m_applicationChanged == null)
				{
					this.CleanContactAppEventVars();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() successful.");
		}

		[SecurityCritical]
		internal void CleanContactAppEventVars()
		{
			CollaborationHelperFunctions.CleanEventVars(ref this.m_regAppChangedWaitHandle, ref this.m_safeAppChangedEvent, ref this.m_appChangedEvent);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ApplicationChanged variables successful.");
		}

		protected virtual void OnApplicationChanged(ApplicationChangedEventArgs appChangedArgs)
		{
			EventHandler<ApplicationChangedEventArgs> applicationChanged = this.m_applicationChanged;
			if (applicationChanged != null)
			{
				if (base.SynchronizingObject != null && base.SynchronizingObject.InvokeRequired)
				{
					base.SynchronizingObject.BeginInvoke(applicationChanged, new object[]
					{
						this,
						appChangedArgs
					});
				}
				else
				{
					applicationChanged(this, appChangedArgs);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the application changed event callback.");
			}
		}

		[SecurityCritical]
		internal virtual void ApplicationChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ApplicationChangedCallback() called.");
			if (!this.m_Disposed)
			{
				for (;;)
				{
					ApplicationChangedEventArgs applicationChangedEventArgs = null;
					try
					{
						lock (this.LockAppChangedEvent)
						{
							if (this.m_safeAppChangedEvent.IsInvalid)
							{
								return;
							}
							num = UnsafeCollabNativeMethods.PeerCollabGetEventData(this.m_safeAppChangedEvent, out safeCollabData);
						}
						if (num == 6488066)
						{
							break;
						}
						if (num != 0)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
							{
								num
							});
							throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetApplicationChangedDataFailed"), num);
						}
						PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointApplicationChanged)
						{
							PEER_EVENT_APPLICATION_CHANGED_DATA applicationChangedData = peer_COLLAB_EVENT_DATA.applicationChangedData;
							PeerContact peerContact = null;
							if (applicationChangedData.pContact != IntPtr.Zero)
							{
								PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(applicationChangedData.pContact, typeof(PEER_CONTACT));
								peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
							}
							if (peerContact != null && this.Equals(peerContact))
							{
								PEER_APPLICATION pa = (PEER_APPLICATION)Marshal.PtrToStructure(applicationChangedData.pApplication, typeof(PEER_APPLICATION));
								PeerApplication peerApplication = CollaborationHelperFunctions.ConvertPEER_APPLICATIONToPeerApplication(pa);
								PeerEndPoint peerEndPoint = null;
								if (applicationChangedData.pEndPoint != IntPtr.Zero)
								{
									PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(applicationChangedData.pEndPoint, typeof(PEER_ENDPOINT));
									peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
								}
								applicationChangedEventArgs = new ApplicationChangedEventArgs(peerEndPoint, peerContact, applicationChangedData.changeType, peerApplication);
							}
						}
					}
					finally
					{
						if (safeCollabData != null)
						{
							safeCollabData.Dispose();
						}
					}
					if (applicationChangedEventArgs != null)
					{
						this.OnApplicationChanged(applicationChangedEventArgs);
					}
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ApplicationChangedCallback().");
			}
		}

		private event EventHandler<ObjectChangedEventArgs> m_objectChanged;

		public event EventHandler<ObjectChangedEventArgs> ObjectChanged
		{
			[SecurityCritical]
			add
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.AddObjectChangedEvent(value);
			}
			[SecurityCritical]
			remove
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.RemoveObjectChangedEvent(value);
			}
		}

		internal object LockObjChangedEvent
		{
			get
			{
				if (this.m_lockObjChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref this.m_lockObjChangedEvent, value, null);
				}
				return this.m_lockObjChangedEvent;
			}
		}

		internal RegisteredWaitHandle ObjChangedWaitHandle
		{
			set
			{
				this.m_regObjChangedWaitHandle = value;
			}
		}

		internal AutoResetEvent ObjChangedEvent
		{
			get
			{
				return this.m_objChangedEvent;
			}
			set
			{
				this.m_objChangedEvent = value;
			}
		}

		[SecurityCritical]
		internal virtual void AddObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddObjectChangedEvent().");
			lock (this.LockObjChangedEvent)
			{
				if (this.m_objectChanged == null)
				{
					this.m_objChangedEvent = new AutoResetEvent(false);
					this.ObjChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(this.m_objChangedEvent, new WaitOrTimerCallback(this.ObjectChangedCallback), null, -1, false);
					PEER_COLLAB_EVENT_REGISTRATION peer_COLLAB_EVENT_REGISTRATION = default(PEER_COLLAB_EVENT_REGISTRATION);
					peer_COLLAB_EVENT_REGISTRATION.eventType = PeerCollabEventType.EndPointObjectChanged;
					peer_COLLAB_EVENT_REGISTRATION.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(this.m_objChangedEvent.SafeWaitHandle, 1U, ref peer_COLLAB_EVENT_REGISTRATION, out this.m_safeObjChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ObjectChangedRegFailed"), num);
					}
				}
				this.m_objectChanged = (EventHandler<ObjectChangedEventArgs>)Delegate.Combine(this.m_objectChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddObjectChanged() successful.");
		}

		[SecurityCritical]
		internal virtual void RemoveObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChangedEvent() called.");
			lock (this.LockObjChangedEvent)
			{
				this.m_objectChanged = (EventHandler<ObjectChangedEventArgs>)Delegate.Remove(this.m_objectChanged, callback);
				if (this.m_objectChanged == null)
				{
					this.CleanContactObjEventVars();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChangedEvent() successful.");
		}

		[SecurityCritical]
		internal void CleanContactObjEventVars()
		{
			CollaborationHelperFunctions.CleanEventVars(ref this.m_regObjChangedWaitHandle, ref this.m_safeObjChangedEvent, ref this.m_objChangedEvent);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ObjectChangedEvent variables successful.");
		}

		protected virtual void OnObjectChanged(ObjectChangedEventArgs objChangedArgs)
		{
			EventHandler<ObjectChangedEventArgs> objectChanged = this.m_objectChanged;
			if (objectChanged != null)
			{
				if (base.SynchronizingObject != null && base.SynchronizingObject.InvokeRequired)
				{
					base.SynchronizingObject.BeginInvoke(objectChanged, new object[]
					{
						this,
						objChangedArgs
					});
				}
				else
				{
					objectChanged(this, objChangedArgs);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the object changed event callback.");
			}
		}

		[SecurityCritical]
		internal virtual void ObjectChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ObjectChangedCallback() called.");
			if (!this.m_Disposed)
			{
				for (;;)
				{
					ObjectChangedEventArgs objectChangedEventArgs = null;
					try
					{
						lock (this.LockObjChangedEvent)
						{
							if (this.m_safeObjChangedEvent.IsInvalid)
							{
								return;
							}
							num = UnsafeCollabNativeMethods.PeerCollabGetEventData(this.m_safeObjChangedEvent, out safeCollabData);
						}
						if (num == 6488066)
						{
							break;
						}
						if (num != 0)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
							{
								num
							});
							throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetObjectChangedDataFailed"), num);
						}
						PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointObjectChanged)
						{
							PEER_EVENT_OBJECT_CHANGED_DATA objectChangedData = peer_COLLAB_EVENT_DATA.objectChangedData;
							PeerContact peerContact = null;
							if (objectChangedData.pContact != IntPtr.Zero)
							{
								PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(objectChangedData.pContact, typeof(PEER_CONTACT));
								peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
							}
							if (peerContact != null && this.Equals(peerContact))
							{
								PeerEndPoint peerEndPoint = null;
								if (objectChangedData.pEndPoint != IntPtr.Zero)
								{
									PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(objectChangedData.pEndPoint, typeof(PEER_ENDPOINT));
									peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
								}
								PEER_OBJECT po = (PEER_OBJECT)Marshal.PtrToStructure(objectChangedData.pObject, typeof(PEER_OBJECT));
								PeerObject peerObject = CollaborationHelperFunctions.ConvertPEER_OBJECTToPeerObject(po);
								objectChangedEventArgs = new ObjectChangedEventArgs(peerEndPoint, peerContact, objectChangedData.changeType, peerObject);
							}
						}
					}
					finally
					{
						if (safeCollabData != null)
						{
							safeCollabData.Dispose();
						}
					}
					if (objectChangedEventArgs != null)
					{
						this.OnObjectChanged(objectChangedEventArgs);
					}
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ObjectChangedCallback().");
			}
		}

		private event EventHandler<PresenceChangedEventArgs> m_presenceChanged;

		public event EventHandler<PresenceChangedEventArgs> PresenceChanged
		{
			[SecurityCritical]
			add
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.AddPresenceChanged(value);
			}
			[SecurityCritical]
			remove
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.RemovePresenceChanged(value);
			}
		}

		internal object LockPresenceChangedEvent
		{
			get
			{
				if (this.m_lockPresenceChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref this.m_lockPresenceChangedEvent, value, null);
				}
				return this.m_lockPresenceChangedEvent;
			}
		}

		internal RegisteredWaitHandle PresenceChangedWaitHandle
		{
			set
			{
				this.m_regPresenceChangedWaitHandle = value;
			}
		}

		internal AutoResetEvent PresenceChangedEvent
		{
			get
			{
				return this.m_presenceChangedEvent;
			}
			set
			{
				this.m_presenceChangedEvent = value;
			}
		}

		[SecurityCritical]
		internal virtual void AddPresenceChanged(EventHandler<PresenceChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddPresenceChanged().");
			lock (this.LockPresenceChangedEvent)
			{
				if (this.m_presenceChanged == null)
				{
					this.m_presenceChangedEvent = new AutoResetEvent(false);
					this.PresenceChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(this.m_presenceChangedEvent, new WaitOrTimerCallback(this.PresenceChangedCallback), null, -1, false);
					PEER_COLLAB_EVENT_REGISTRATION peer_COLLAB_EVENT_REGISTRATION = default(PEER_COLLAB_EVENT_REGISTRATION);
					peer_COLLAB_EVENT_REGISTRATION.eventType = PeerCollabEventType.EndPointPresenceChanged;
					peer_COLLAB_EVENT_REGISTRATION.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(this.m_presenceChangedEvent.SafeWaitHandle, 1U, ref peer_COLLAB_EVENT_REGISTRATION, out this.m_safePresenceChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_PresenceChangedRegFailed"), num);
					}
				}
				this.m_presenceChanged = (EventHandler<PresenceChangedEventArgs>)Delegate.Combine(this.m_presenceChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddPresenceChanged() successful.");
		}

		[SecurityCritical]
		internal virtual void RemovePresenceChanged(EventHandler<PresenceChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePresenceChanged() called.");
			lock (this.LockPresenceChangedEvent)
			{
				this.m_presenceChanged = (EventHandler<PresenceChangedEventArgs>)Delegate.Remove(this.m_presenceChanged, callback);
				if (this.m_presenceChanged == null)
				{
					this.CleanContactPresenceEventVars();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePresenceChanged() successful.");
		}

		[SecurityCritical]
		internal void CleanContactPresenceEventVars()
		{
			CollaborationHelperFunctions.CleanEventVars(ref this.m_regPresenceChangedWaitHandle, ref this.m_safePresenceChangedEvent, ref this.m_presenceChangedEvent);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean PresenceChanged variables successful.");
		}

		protected virtual void OnPresenceChanged(PresenceChangedEventArgs presenceChangedArgs)
		{
			EventHandler<PresenceChangedEventArgs> presenceChanged = this.m_presenceChanged;
			if (presenceChanged != null)
			{
				if (base.SynchronizingObject != null && base.SynchronizingObject.InvokeRequired)
				{
					base.SynchronizingObject.BeginInvoke(presenceChanged, new object[]
					{
						this,
						presenceChangedArgs
					});
				}
				else
				{
					presenceChanged(this, presenceChangedArgs);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the presence changed event callback.");
			}
		}

		[SecurityCritical]
		internal virtual void PresenceChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PresenceChangedCallback() called.");
			if (!this.m_Disposed)
			{
				for (;;)
				{
					PresenceChangedEventArgs presenceChangedEventArgs = null;
					try
					{
						lock (this.LockPresenceChangedEvent)
						{
							if (this.m_safePresenceChangedEvent.IsInvalid)
							{
								return;
							}
							num = UnsafeCollabNativeMethods.PeerCollabGetEventData(this.m_safePresenceChangedEvent, out safeCollabData);
						}
						if (num == 6488066)
						{
							break;
						}
						if (num != 0)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
							{
								num
							});
							throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetPresenceChangedDataFailed"), num);
						}
						PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointPresenceChanged)
						{
							PEER_EVENT_PRESENCE_CHANGED_DATA presenceChangedData = peer_COLLAB_EVENT_DATA.presenceChangedData;
							PeerContact peerContact = null;
							if (presenceChangedData.pContact != IntPtr.Zero)
							{
								PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(presenceChangedData.pContact, typeof(PEER_CONTACT));
								peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
							}
							if (peerContact != null && this.Equals(peerContact))
							{
								PeerEndPoint peerEndPoint = null;
								if (presenceChangedData.pEndPoint != IntPtr.Zero)
								{
									PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(presenceChangedData.pEndPoint, typeof(PEER_ENDPOINT));
									peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
								}
								PeerPresenceInfo peerPresenceInfo = null;
								if (presenceChangedData.pPresenceInfo != IntPtr.Zero)
								{
									PEER_PRESENCE_INFO peer_PRESENCE_INFO = (PEER_PRESENCE_INFO)Marshal.PtrToStructure(presenceChangedData.pPresenceInfo, typeof(PEER_PRESENCE_INFO));
									peerPresenceInfo = new PeerPresenceInfo();
									peerPresenceInfo.PresenceStatus = peer_PRESENCE_INFO.status;
									peerPresenceInfo.DescriptiveText = peer_PRESENCE_INFO.descText;
								}
								presenceChangedEventArgs = new PresenceChangedEventArgs(peerEndPoint, peerContact, presenceChangedData.changeType, peerPresenceInfo);
							}
						}
					}
					finally
					{
						if (safeCollabData != null)
						{
							safeCollabData.Dispose();
						}
					}
					if (presenceChangedEventArgs != null)
					{
						this.OnPresenceChanged(presenceChangedEventArgs);
					}
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving PresenceChangedCallback().");
			}
		}

		public bool Equals(PeerContact other)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			if (other != null)
			{
				if (other.PeerName != null)
				{
					return other.PeerName.Equals(this.PeerName);
				}
				if (this.PeerName == null)
				{
					return true;
				}
			}
			return false;
		}

		public override bool Equals(object obj)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerContact peerContact = obj as PeerContact;
			return peerContact != null && this.Equals(peerContact);
		}

		public new static bool Equals(object objA, object objB)
		{
			PeerContact peerContact = objA as PeerContact;
			PeerContact peerContact2 = objB as PeerContact;
			if (peerContact != null && peerContact2 != null)
			{
				if (peerContact.PeerName != null)
				{
					return peerContact.PeerName.Equals(peerContact2.PeerName);
				}
				if (peerContact2.PeerName == null)
				{
					return true;
				}
			}
			return false;
		}

		public override int GetHashCode()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			return (this.PeerName == null) ? 0 : this.PeerName.GetHashCode();
		}

		public override string ToString()
		{
			return this.m_displayName;
		}

		[SecurityCritical]
		protected override void Dispose(bool disposing)
		{
			if (!this.m_Disposed)
			{
				try
				{
					this.CleanContactAppEventVars();
					this.CleanContactObjEventVars();
					this.CleanContactPresenceEventVars();
					this.m_Disposed = true;
				}
				finally
				{
					base.Dispose(disposing);
				}
			}
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			this.GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_PeerName", this.m_peerName);
			info.AddValue("_NickName", this.m_nickname);
			info.AddValue("_DisplayName", this.m_displayName);
			if (this.m_emailAddress != null)
			{
				info.AddValue("_EmailAddress", this.m_emailAddress.ToString());
			}
			info.AddValue("_SubscribeAllowed", this.m_subscribeAllowed);
			info.AddValue("_Credentials", this.m_credentials.RawData);
			info.AddValue("_ContactXml", this.m_contactXml);
			info.AddValue("_JustCreated", this.m_justCreated);
		}

		internal void TracePeerContact()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Contents of the PeerContact");
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tPeerName: {0}", new object[]
			{
				this.PeerName
			});
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tNickname: {0}", new object[]
			{
				this.Nickname
			});
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tEmailAddress: {0}", new object[]
			{
				this.EmailAddress.ToString()
			});
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tSubscribeAllowed: {0}", new object[]
			{
				this.SubscribeAllowed
			});
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tCredentials FriendlyName: {0}", new object[]
			{
				this.Credentials.FriendlyName
			});
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Verbose))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tCredentials raw data:");
				Logging.DumpData(Logging.P2PTraceSource, TraceEventType.Verbose, Logging.P2PTraceSource.MaxDataSize, this.Credentials.RawData, 0, this.Credentials.RawData.Length);
			}
			else
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tCredentials raw data length {0}", new object[]
				{
					this.Credentials.RawData.Length
				});
			}
		}

		private PeerName m_peerName;

		private string m_nickname;

		private string m_displayName;

		private MailAddress m_emailAddress;

		private SubscriptionType m_subscribeAllowed;

		private bool m_isSubscribed;

		private X509Certificate2 m_credentials;

		private string m_contactXml;

		private bool m_justCreated;

		private object m_isSubscribeLock;

		private AsyncOperation m_AsyncOp;

		private object m_asyncOpLock;

		private SendOrPostCallback OnSubscribeCompletedDelegate;

		private object m_lockAppChangedEvent;

		private RegisteredWaitHandle m_regAppChangedWaitHandle;

		private AutoResetEvent m_appChangedEvent;

		internal SafeCollabEvent m_safeAppChangedEvent;

		private object m_lockObjChangedEvent;

		private RegisteredWaitHandle m_regObjChangedWaitHandle;

		private AutoResetEvent m_objChangedEvent;

		internal SafeCollabEvent m_safeObjChangedEvent;

		private object m_lockPresenceChangedEvent;

		private RegisteredWaitHandle m_regPresenceChangedWaitHandle;

		internal SafeCollabEvent m_safePresenceChangedEvent;

		private bool m_Disposed;
	}
}
