﻿using System;
using System.Runtime.InteropServices;
using System.Security;
using System.Security.Permissions;
using Microsoft.Win32.SafeHandles;

namespace System.Net.PeerToPeer.Collaboration
{
	[SuppressUnmanagedCodeSecurity]
	[SecurityCritical(SecurityCriticalScope.Everything)]
	internal sealed class SafeCollabMemory : SafeHandleZeroOrMinusOneIsInvalid
	{
		internal SafeCollabMemory() : base(true)
		{
		}

		[SecurityPermission(SecurityAction.LinkDemand, UnmanagedCode = true)]
		internal SafeCollabMemory(int cb) : base(true)
		{
			this.handle = Marshal.AllocHGlobal(cb);
			if (object.Equals(this.handle, IntPtr.Zero))
			{
				base.SetHandleAsInvalid();
				throw new PeerToPeerException(SR.GetString("MemoryAllocFailed"));
			}
			this.allocated = true;
		}

		protected override bool ReleaseHandle()
		{
			if (this.allocated && !this.IsInvalid)
			{
				Marshal.FreeHGlobal(this.handle);
			}
			base.SetHandleAsInvalid();
			base.SetHandle(IntPtr.Zero);
			return true;
		}

		private bool allocated;
	}
}
