﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;
using System.Threading;

namespace System.Net.PeerToPeer
{
	[Serializable]
	public class PeerToPeerException : Exception, ISerializable
	{
		public PeerToPeerException()
		{
		}

		public PeerToPeerException(string message) : base(message)
		{
		}

		public PeerToPeerException(string message, Exception innerException) : base(message, innerException)
		{
		}

		[SecurityCritical]
		internal static PeerToPeerException CreateFromHr(string message, int hr)
		{
			PeerToPeerException ex = null;
			int num = hr >> 16 & 8191;
			IntPtr zero = IntPtr.Zero;
			try
			{
				uint num2 = UnsafeSystemNativeMethods.FormatMessage(FormatMessageFlags.FORMAT_MESSAGE_ALLOCATE_BUFFER | FormatMessageFlags.FORMAT_MESSAGE_ARGUMENT_ARRAY | (((long)num == 99L) ? FormatMessageFlags.FORMAT_MESSAGE_FROM_HMODULE : FormatMessageFlags.FORMAT_MESSAGE_FROM_SYSTEM), ((long)num == 99L) ? PeerToPeerOSHelper.P2PModuleHandle : IntPtr.Zero, (uint)hr, 0U, ref zero, 0U, IntPtr.Zero);
				if (num2 != 0U)
				{
					string message2 = Marshal.PtrToStringUni(zero);
					ex = new PeerToPeerException(message, new Win32Exception(hr, message2));
				}
				else
				{
					ex = new PeerToPeerException(message, Marshal.GetExceptionForHR(hr));
				}
			}
			catch (Exception ex2)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Warning, 0, "Could not get the error message for error code {0} - Exception {1}", new object[]
				{
					hr,
					ex2
				});
				if (ex2 is ThreadAbortException || ex2 is StackOverflowException || ex2 is OutOfMemoryException)
				{
					throw;
				}
			}
			finally
			{
				if (zero != IntPtr.Zero)
				{
					UnsafeSystemNativeMethods.LocalFree(zero);
				}
			}
			if (ex == null)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Warning, 0, "Could not get the error message for error code {0}", new object[]
				{
					hr
				});
				ex = new PeerToPeerException(message + "Underlying native error " + hr);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "Exception: {0}", new object[]
			{
				ex
			});
			return ex;
		}

		protected PeerToPeerException(SerializationInfo info, StreamingContext context) : base(info, context)
		{
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			this.GetObjectData(info, context);
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter)]
		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);
		}

		private const uint FACILITY_P2P = 99U;
	}
}
